@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,29 +32,16 @@
 
 #pragma once
 
-#include "BufferCompressor.h"
+#include "DeflateBaseCompressor.h"
 
 namespace Orthanc
 {
-  class ZlibCompressor : public BufferCompressor
+  class ZlibCompressor : public DeflateBaseCompressor
   {
-  private:
-    uint8_t compressionLevel_;
-
   public:
-    using BufferCompressor::Compress;
-    using BufferCompressor::Uncompress;
-
     ZlibCompressor()
     {
-      compressionLevel_ = 6;
-    }
-
-    void SetCompressionLevel(uint8_t level);
-
-    uint8_t GetCompressionLevel() const
-    {
-      return compressionLevel_;
+      SetPrefixWithUncompressedSize(true);
     }
 
     virtual void Compress(std::string& compressed,
