@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,6 +34,8 @@
 
 #include "DicomMap.h"
 
+#include "DicomImageInformation.h"
+
 #include <stdint.h>
 
 namespace Orthanc
@@ -41,20 +43,14 @@ namespace Orthanc
   class DicomIntegerPixelAccessor
   {
   private:
-    unsigned int width_;
-    unsigned int height_;
-    unsigned int samplesPerPixel_;
-    unsigned int numberOfFrames_;
-    unsigned int planarConfiguration_;
-    const void* pixelData_;
-    size_t size_;
+    DicomImageInformation information_;
 
-    uint8_t shift_;
     uint32_t signMask_;
     uint32_t mask_;
-    size_t bytesPerPixel_;
-    unsigned int frame_;
 
+    const void* pixelData_;
+    size_t size_;
+    unsigned int frame_;
     size_t frameOffset_;
     size_t rowOffset_;
 
@@ -63,19 +59,9 @@ namespace Orthanc
                               const void* pixelData,
                               size_t size);
 
-    unsigned int GetWidth() const
+    const DicomImageInformation GetInformation() const
     {
-      return width_;
-    }
-
-    unsigned int GetHeight() const
-    {
-      return height_;
-    }
-
-    unsigned int GetNumberOfFrames() const
-    {
-      return numberOfFrames_;
+      return information_;
     }
 
     unsigned int GetCurrentFrame() const
@@ -88,11 +74,16 @@ namespace Orthanc
     void GetExtremeValues(int32_t& min, 
                           int32_t& max) const;
 
-    unsigned int GetChannelCount() const
+    int32_t GetValue(unsigned int x, unsigned int y, unsigned int channel = 0) const;
+
+    const void* GetPixelData() const
     {
-      return samplesPerPixel_;
+      return pixelData_;
     }
 
-    int32_t GetValue(unsigned int x, unsigned int y, unsigned int channel = 0) const;
+    size_t GetSize() const
+    {
+      return size_;
+    }
   };
 }
