@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,44 +32,65 @@
 
 #pragma once
 
+#include "IStorageArea.h"
 #include "FileInfo.h"
-#include "../HttpServer/HttpFileSender.h"
+#include "../HttpServer/BufferHttpSender.h"
+#include "../RestApi/RestApiOutput.h"
 
 #include <vector>
 #include <string>
 #include <boost/noncopyable.hpp>
 #include <stdint.h>
+#include <json/value.h>
 
 namespace Orthanc
 {
   class StorageAccessor : boost::noncopyable
   {
-  protected:
-    virtual FileInfo WriteInternal(const void* data,
-                                   size_t size,
-                                   FileContentType type) = 0;
+  private:
+    IStorageArea&  area_;
+
+    void SetupSender(BufferHttpSender& sender,
+                     const FileInfo& info,
+                     const std::string& mime);
 
   public:
-    virtual ~StorageAccessor()
+    StorageAccessor(IStorageArea& area) : area_(area)
     {
     }
 
     FileInfo Write(const void* data,
                    size_t size,
-                   FileContentType type)
+                   FileContentType type,
+                   CompressionType compression,
+                   bool storeMd5);
+
+    FileInfo Write(const std::string& data, 
+                   FileContentType type,
+                   CompressionType compression,
+                   bool storeMd5)
     {
-      return WriteInternal(data, size, type);
+      return Write((data.size() == 0 ? NULL : data.c_str()),
+                   data.size(), type, compression, storeMd5);
     }
 
-    FileInfo Write(const std::vector<uint8_t>& content,
-                   FileContentType type);
+    void Read(std::string& content,
+              const FileInfo& info);
 
-    FileInfo Write(const std::string& content,
-                   FileContentType type);
+    void Read(Json::Value& content,
+              const FileInfo& info);
 
-    virtual void Read(std::string& content,
-                      const std::string& uuid) = 0;
+    void Remove(const FileInfo& info)
+    {
+      area_.Remove(info.GetUuid(), info.GetContentType());
+    }
 
-    virtual HttpFileSender* ConstructHttpFileSender(const std::string& uuid) = 0;
+    void AnswerFile(HttpOutput& output,
+                    const FileInfo& info,
+                    const std::string& mime);
+
+    void AnswerFile(RestApiOutput& output,
+                    const FileInfo& info,
+                    const std::string& mime);
   };
 }
