@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,9 +32,11 @@
 
 #pragma once
 
+#include "../IDynamicObject.h"
+
 namespace Orthanc
 {
-  class IRunnableBySteps
+  class IRunnableBySteps : public IDynamicObject
   {
   public:
     virtual ~IRunnableBySteps()
