@@ -1,7 +1,8 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ *
+ * Copyright (C) 2012-2015 Sebastien Jodogne <s.jodogne@gmail.com>,
+ * Medical Physics Department, CHU of Liege, Belgium
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -31,9 +32,16 @@
  **/
 
 
-#include "FunctionContext.h"
+#if ORTHANC_SQLITE_STANDALONE != 1
+#include "../PrecompiledHeaders.h"
+#endif
 
-#include <sqlite3.h>
+#include "FunctionContext.h"
+#include "OrthancSQLiteException.h"
+
+#include <string>
+
+#include "sqlite3.h"
 
 namespace Orthanc
 {
@@ -56,7 +64,7 @@ namespace Orthanc
     {
       if (index >= argc_)
       {
-        throw OrthancException(ErrorCode_ParameterOutOfRange);
+        throw OrthancSQLiteException(ErrorCode_ParameterOutOfRange);
       }
     }
 
@@ -89,6 +97,12 @@ namespace Orthanc
       CheckIndex(index);
       return std::string(reinterpret_cast<const char*>(sqlite3_value_text(argv_[index])));
     }
+
+    bool FunctionContext::IsNullValue(unsigned int index) const
+    {
+      CheckIndex(index);
+      return sqlite3_value_type(argv_[index]) == SQLITE_NULL;
+    }
   
     void FunctionContext::SetNullResult()
     {
