@@ -1,7 +1,8 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ *
+ * Copyright (C) 2012-2015 Sebastien Jodogne <s.jodogne@gmail.com>,
+ * Medical Physics Department, CHU of Liege, Belgium
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,13 +34,14 @@
 
 #pragma once
 
+#include "NonCopyable.h"
 #include "FunctionContext.h"
 
 namespace Orthanc
 {
   namespace SQLite
   {
-    class IScalarFunction : public boost::noncopyable
+    class IScalarFunction : public NonCopyable
     {
     public:
       virtual ~IScalarFunction()
