@@ -1,7 +1,8 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ *
+ * Copyright (C) 2012-2015 Sebastien Jodogne <s.jodogne@gmail.com>,
+ * Medical Physics Department, CHU of Liege, Belgium
  *
  * Copyright (c) 2012 The Chromium Authors. All rights reserved.
  *
@@ -34,41 +35,64 @@
  **/
 
 
+#if ORTHANC_SQLITE_STANDALONE != 1
+#include "../PrecompiledHeaders.h"
+#endif
+
 #include "Statement.h"
-
 #include "Connection.h"
-#include "../Toolbox.h"
 
-#include <boost/lexical_cast.hpp>
-#include <sqlite3.h>
 #include <string.h>
-#include <glog/logging.h>
+#include <stdio.h>
+#include <algorithm>
+
+#if ORTHANC_SQLITE_STANDALONE != 1
+#include "../Logging.h"
+#endif
+
+#include "sqlite3.h"
+
+#if defined(_MSC_VER)
+#define snprintf _snprintf
+#endif
 
 namespace Orthanc
 {
   namespace SQLite
   {
-    int Statement::CheckError(int err) const
+    int Statement::CheckError(int err, ErrorCode code) const
     {
       bool succeeded = (err == SQLITE_OK || err == SQLITE_ROW || err == SQLITE_DONE);
       if (!succeeded)
       {
-        throw OrthancException("SQLite error code " + boost::lexical_cast<std::string>(err));
+#if ORTHANC_SQLITE_STANDALONE != 1
+        char buffer[128];
+        snprintf(buffer, sizeof(buffer) - 1, "SQLite error code %d", err);
+        LOG(ERROR) << buffer;
+#endif
+
+        throw OrthancSQLiteException(code);
       }
 
       return err;
     }
 
-    void Statement::CheckOk(int err) const 
+    void Statement::CheckOk(int err, ErrorCode code) const 
     {
       if (err == SQLITE_RANGE)
       {
         // Binding to a non-existent variable is evidence of a serious error.
-        throw OrthancException("Bind value out of range");
+        throw OrthancSQLiteException(ErrorCode_SQLiteBindOutOfRange);
       }
       else if (err != SQLITE_OK)
       {
-        throw OrthancException("SQLite error code " + boost::lexical_cast<std::string>(err));
+#if ORTHANC_SQLITE_STANDALONE != 1
+        char buffer[128];
+        snprintf(buffer, sizeof(buffer) - 1, "SQLite error code %d", err);
+        LOG(ERROR) << buffer;
+#endif
+
+        throw OrthancSQLiteException(code);
       }
     }
 
@@ -107,14 +131,20 @@ namespace Orthanc
 
     bool Statement::Run()
     {
+#if ORTHANC_SQLITE_STANDALONE != 1
       VLOG(1) << "SQLite::Statement::Run " << sqlite3_sql(GetStatement());
-      return CheckError(sqlite3_step(GetStatement())) == SQLITE_DONE;
+#endif
+
+      return CheckError(sqlite3_step(GetStatement()), ErrorCode_SQLiteCannotRun) == SQLITE_DONE;
     }
 
     bool Statement::Step()
     {
+#if ORTHANC_SQLITE_STANDALONE != 1
       VLOG(1) << "SQLite::Statement::Step " << sqlite3_sql(GetStatement());
-      return CheckError(sqlite3_step(GetStatement())) == SQLITE_ROW;
+#endif
+
+      return CheckError(sqlite3_step(GetStatement()), ErrorCode_SQLiteCannotStep) == SQLITE_ROW;
     }
 
     void Statement::Reset(bool clear_bound_vars) 
@@ -136,7 +166,8 @@ namespace Orthanc
 
     void Statement::BindNull(int col)
     {
-      CheckOk(sqlite3_bind_null(GetStatement(), col + 1));
+      CheckOk(sqlite3_bind_null(GetStatement(), col + 1),
+              ErrorCode_BadParameterType);
     }
 
     void Statement::BindBool(int col, bool val) 
@@ -146,22 +177,26 @@ namespace Orthanc
 
     void Statement::BindInt(int col, int val) 
     {
-      CheckOk(sqlite3_bind_int(GetStatement(), col + 1, val));
+      CheckOk(sqlite3_bind_int(GetStatement(), col + 1, val),
+              ErrorCode_BadParameterType);
     }
 
     void Statement::BindInt64(int col, int64_t val) 
     {
-      CheckOk(sqlite3_bind_int64(GetStatement(), col + 1, val));
+      CheckOk(sqlite3_bind_int64(GetStatement(), col + 1, val),
+              ErrorCode_BadParameterType);
     }
 
     void Statement::BindDouble(int col, double val) 
     {
-      CheckOk(sqlite3_bind_double(GetStatement(), col + 1, val));
+      CheckOk(sqlite3_bind_double(GetStatement(), col + 1, val),
+              ErrorCode_BadParameterType);
     }
 
     void Statement::BindCString(int col, const char* val) 
     {
-      CheckOk(sqlite3_bind_text(GetStatement(), col + 1, val, -1, SQLITE_TRANSIENT));
+      CheckOk(sqlite3_bind_text(GetStatement(), col + 1, val, -1, SQLITE_TRANSIENT),
+              ErrorCode_BadParameterType);
     }
 
     void Statement::BindString(int col, const std::string& val) 
@@ -170,7 +205,8 @@ namespace Orthanc
                                 col + 1,
                                 val.data(),
                                 val.size(),
-                                SQLITE_TRANSIENT));
+                                SQLITE_TRANSIENT),
+              ErrorCode_BadParameterType);
     }
 
     /*void Statement::BindString16(int col, const string16& value) 
@@ -180,7 +216,8 @@ namespace Orthanc
 
     void Statement::BindBlob(int col, const void* val, int val_len) 
     {
-      CheckOk(sqlite3_bind_blob(GetStatement(), col + 1, val, val_len, SQLITE_TRANSIENT));
+      CheckOk(sqlite3_bind_blob(GetStatement(), col + 1, val, val_len, SQLITE_TRANSIENT),
+              ErrorCode_BadParameterType);
     }
 
 
@@ -205,7 +242,7 @@ namespace Orthanc
     ColumnType Statement::GetDeclaredColumnType(int col) const 
     {
       std::string column_type(sqlite3_column_decltype(GetStatement(), col));
-      Toolbox::ToLowerCase(column_type);
+      std::transform(column_type.begin(), column_type.end(), column_type.begin(), tolower);
 
       if (column_type == "integer")
         return COLUMN_TYPE_INTEGER;
@@ -295,7 +332,7 @@ namespace Orthanc
       return true;
       }*/
 
-    bool Statement::ColumnBlobAsVector(int col, std::vector<char>* val) const 
+    /*bool Statement::ColumnBlobAsVector(int col, std::vector<char>* val) const 
     {
       val->clear();
 
@@ -306,14 +343,14 @@ namespace Orthanc
         memcpy(&(*val)[0], data, len);
       }
       return true;
-    }
+      }*/
 
-    bool Statement::ColumnBlobAsVector(
+    /*bool Statement::ColumnBlobAsVector(
       int col,
       std::vector<unsigned char>* val) const 
     {
       return ColumnBlobAsVector(col, reinterpret_cast< std::vector<char>* >(val));
-    }
+      }*/
 
   }
 }
