@@ -1,7 +1,8 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ *
+ * Copyright (C) 2012-2015 Sebastien Jodogne <s.jodogne@gmail.com>,
+ * Medical Physics Department, CHU of Liege, Belgium
  *
  * Copyright (c) 2012 The Chromium Authors. All rights reserved.
  *
@@ -34,7 +35,12 @@
  **/
 
 
+#if ORTHANC_SQLITE_STANDALONE != 1
+#include "../PrecompiledHeaders.h"
+#endif
+
 #include "Transaction.h"
+#include "OrthancSQLiteException.h"
 
 namespace Orthanc
 {
@@ -58,13 +64,13 @@ namespace Orthanc
     {
       if (isOpen_) 
       {
-        throw OrthancException("SQLite: Beginning a transaction twice!");
+        throw OrthancSQLiteException(ErrorCode_SQLiteTransactionAlreadyStarted);
       }
 
       isOpen_ = connection_.BeginTransaction();
       if (!isOpen_)
       {
-        throw OrthancException("SQLite: Unable to create a transaction");
+        throw OrthancSQLiteException(ErrorCode_SQLiteTransactionBegin);
       }
     }
 
@@ -72,8 +78,7 @@ namespace Orthanc
     {
       if (!isOpen_) 
       {
-        throw OrthancException("SQLite: Attempting to roll back a nonexistent transaction. "
-                                "Did you remember to call Begin()?");
+        throw OrthancSQLiteException(ErrorCode_SQLiteRollbackWithoutTransaction);
       }
 
       isOpen_ = false;
@@ -85,15 +90,14 @@ namespace Orthanc
     {
       if (!isOpen_) 
       {
-        throw OrthancException("SQLite: Attempting to roll back a nonexistent transaction. "
-                                "Did you remember to call Begin()?");
+        throw OrthancSQLiteException(ErrorCode_SQLiteRollbackWithoutTransaction);
       }
 
       isOpen_ = false;
 
       if (!connection_.CommitTransaction())
       {
-        throw OrthancException("SQLite: Failure when committing the transaction");
+        throw OrthancSQLiteException(ErrorCode_SQLiteTransactionCommit);
       }
     }
   }
