@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,6 +43,7 @@ namespace Orthanc
     OFCondition storeScp(T_ASC_Association * assoc, 
                          T_DIMSE_Message * msg, 
                          T_ASC_PresentationContextID presID,
-                         IStoreRequestHandler& handler);
+                         IStoreRequestHandler& handler,
+                         const std::string& remoteIp);
   }
 }
